#!/usr/local/bin/perl
# Create, update or delete an ACL generator

require './generate-lib.pl';
&ReadParse();
if (!$in{'new'}) {
	@acls = &list_acls();
        ($acl) = grep { $_->{'name'} eq $in{'old'} } @acls;
	$oldfile = $acl->{'file'};
	@jobs = &cron::list_cron_jobs();
	($oldjob) = grep { $_->{'command'} eq "$cron_cmd $acl->{'name'}" } @jobs;
	}
else {
	$acl = { };
	}

$conf = &squid::get_config();
@sacls = &squid::find_config("acl", $conf);
foreach $s (@sacls) {
	$taken{$s->{'values'}->[0]} = $s;
	}

if ($in{'delete'}) {
	# Delete from Squid (if possible)
	&error_setup($text{'save_err2'});
	foreach $h (&squid::find_config("http_access", $conf),
		    &squid::find_config("icp_access", $conf)) {
		foreach $v (@{$h->{'values'}}) {
			if ($v eq $acl->{'name'} ||
			    $v eq "!".$acl->{'name'}) {
				&error($text{'save_esquid'});
				}
			}
		}
	@sacls = grep { $_ ne $taken{$acl->{'name'}} } @sacls;
	&squid::save_directive($conf, "acl", \@sacls);
	&flush_file_lines();

	# Delete cron job
	if ($oldjob) {
		&cron::delete_cron_job($oldjob);
		}

	# Delete from this module
	&delete_acl($acl);
	unlink($acl->{'file'});

	&redirect("");
	}
else {
	# Validate inputs
	&error_setup($text{'save_err1'});
	if ($in{'new'}) {
		$in{'name'} =~ /^[a-zA-Z0-9\.\-\_]+$/ || &error($text{'save_ename'});
		$taken{$in{'name'}} && &error($text{'save_eclash'});
		$acl->{'name'} = $in{'name'};
		}
	if ($in{'url'} =~ /^[a-z0-9\.\-\_]+(\/|$)/i) {
		$in{'url'} = "http://$in{'url'}";
		}
	$in{'url'} =~ /^(http:|https:)\/\// || &error($text{'save_eurl'});
	if ($in{'url'} =~ /^(http:|https:)\/\/([^\/]+)$/) {
		$in{'url'} .= "/";
		}
	$acl->{'url'} = $in{'url'};
	$in{'recur'} =~ /^\d+$/ && $in{'recur'} > 0 ||
		&error($text{'save_erecur'});
	$acl->{'recur'} = $in{'recur'};
	if (!$in{'auto'}) {
		$in{'file'} =~ /^\/\S./ || &error($text{'save_efile'});
		$acl->{'file'} = $in{'file'};
		}
	else {
		$squid::config{'squid_conf'} =~ /^(.*)\//;
		$acl->{'file'} = "$1/$in{'name'}.txt";
		}
	$acl->{'sched'} = $in{'sched'};
	&cron::parse_times_input($acl, \%in);

	# Do a test download
	($host, $port, $page, $ssl) = &parse_http_url($in{'url'});
	$temp = &tempname();
	&http_download($host, $port, $page, $temp, \$err, undef, $ssl);
	unlink($temp);
	if ($err) {
		&error(&text('save_edownload', $err));
		}

	# Make sure file can be written to
	if ($in{'new'}) {
		open(FILE, ">>$acl->{'file'}") ||
			&error(&text('save_ewrite', $!));
		close(FILE);
		}
	elsif ($oldfile ne $acl->{'file'}) {
		rename($oldfile, $acl->{'file'}) ||
			&error(&text('save_erename', $!));
		}

	# Update the cron job
	%job = %$oldjob;
	foreach $k ('special', 'mins', 'hours', 'days', 'months', 'weekdays') {
		$job{$k} = $acl->{$k};
		}
	&cron::create_wrapper($cron_cmd, $module_name, "refresh.pl");
	if ($acl->{'sched'} && !$oldjob) {
		# Create cron job
		$job{'user'} = 'root';
		$job{'command'} = "$cron_cmd $acl->{'name'}";
		$job{'active'} = 1;
		&cron::create_cron_job(\%job);
		}
	elsif ($acl->{'sched'} && $oldjob) {
		# Update cron job
		&cron::change_cron_job(\%job);
		}
	elsif (!$acl->{'sched'} && $oldjob) {
		# Delete cron job
		&cron::delete_cron_job(\%job);
		}

	# Save generator
	&save_acl($acl);

	if ($in{'new'}) {
		# Add to Squid configuration
		push(@sacls, { 'name' => 'acl',
			       'values' => [ $in{'name'}, 'dstdomain',
					     "\"$acl->{'file'}\"" ] });
		}
	else {
		# Update filename in Squid configuration
		if ($oldfile ne $acl->{'file'}) {
			$taken{$acl->{'name'}}->{'values'}->[2] =
				"\"$acl->{'file'}\"";
			}
		}
	&squid::save_directive($conf, "acl", \@sacls);
	&flush_file_lines();

	if ($in{'new'} || $in{'refresh'}) {
		# Download whole site now
		$| = 1;
		$theme_no_table++;
		&header($text{'refresh_title'}, "");
		print "<hr>\n";

		if ($in{'new'}) {
			print "$text{'refresh_doing1'}<p>\n";
			}
		else {
			print "$text{'refresh_doing2'}<p>\n";
			}

		$count = &download_acl($acl, \&print_url);
		print "<p>\n";
		if ($count) {
			print &text('refresh_done', $count),"<p>\n";
			}
		else {
			print &text('refresh_failed'),"<p>\n";
			}

		print "<hr>\n";
		&footer("", $text{'index_return'});
		}
	else {
		&redirect("");
		}
	}


sub print_url
{
local $url = length($_[0]) > 80 ? "<tt>".&html_escape(substr($_[0], 0, 80))."</tt>..."
				: "<tt>".&html_escape($_[0])."</tt>";
print &text('refresh_url', $url),"<br>\n";
}

